from pyOwlBoard import client
import os

from gitea import GiteaConnector

## Validates and filters input based on whether diagram time matches schedule start time
def filter_timetable_entries(diagram_entries):
    for entry in diagram_entries:
        diagram_time1 = (entry.get('diagram_time1') or '').replace('.', '')
        diagram_time2 = (entry.get('diagram_time2') or '').replace('.', '')
        timetable_entries = entry.get('timetable_entries', [])

        # Collect train start times for comparison
        train_start_times = [t['trainStartTime'] for t in timetable_entries]

        # Check for match
        matching_times = [
            t for t in train_start_times
            if t == diagram_time1 or t== diagram_time2
        ]

        # Check for unique matches
        if len(matching_times) == 1:
            # Filter any non-matching entries if there is a match
            entry['timetable_entries'] = [
                t for t in timetable_entries
                if t['trainStartTime'] == matching_times[0]
            ]

            entry['verification'] = "AUTO: Diagram/Schedule Time Match"

    return diagram_entries

## Checks OwlBoard API for existing PIS codes and whether they match
def check_and_validate_against_owlboard(train_entries):
    ob_client = client.OwlBoardClient("https://owlboard.info", os.getenv("OWLBOARD_KEY"))
    gitea_client = GiteaConnector("https://git.fjla.uk/api/v1/repos/owlboard/data/", "fred.boniface", os.getenv("GITEA_PASS"))
    
    output = []
    for train_entry in train_entries:
        existing_pis = ob_client.get_stops_by_pis(train_entry['diagram_pis_code'])
        if existing_pis:
            database_stops = [stop.upper() for stop in existing_pis[0].get("stops", [])]
        else:
            database_stops = []
        
        ## If only one possible entry, and it matches stops in Database, continue
        if (
            len(train_entry['timetable_entries']) == 1
            and database_stops == train_entry['timetable_entries'][0]["stops"]
        ): continue

        ## If database stops are empty (not in database) add to output list
        elif not database_stops:
            output.append(train_entry)

        # Else if only one possible entry (and previous statements false), open issue
        elif len(train_entry['timetable_entries']) == 1 and train_entry['diagram_pis_code'] != "____":
            issue_title = f"PIS Error | Code: {train_entry['diagram_pis_code']}"
            issue_content = f"""
PIS Code {train_entry['diagram_pis_code']}.
Diagram dated {train_entry['diagram_date']} has been parsed and produced a mismatch with the OwlBoard database.

Diagram  Stops: {",".join(train_entry['timetable_entries'][0]["stops"])}

Database Stops: {",".join(database_stops)}

The train which was identified in the database is:
stpIndicator: {train_entry['timetable_entries'][0]['stpIndicator']}
operator: {train_entry['timetable_entries'][0]['operator']}
trainUID: {train_entry['timetable_entries'][0]['trainUid']}
scheduleStart: {train_entry['timetable_entries'][0]['scheduleStart']}
scheduleEnd: {train_entry['timetable_entries'][0]['scheduleEnd']}
daysRun: {",".join(train_entry['timetable_entries'][0]['daysRun'])}

This requires a manual check of the PIS file to identify mistakes or changes.
Make sure that you validate that the service runs on the correct day.
If the stpIndicator is 'O' or 'N', it is possible that the train schedule is newer than the parsed diagram.
            """
            print(issue_content)
            gitea_client.create_issue(issue_title, issue_content)
            continue

        ## Else append to output
        else:
            output.append(train_entry)
    return output
